package schedframe.scheduling;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import schedframe.resources.StandardResourceType;
import schedframe.resources.computing.description.ExecutingResourceDescription;
import schedframe.resources.computing.description.ResourceDescription;
import schedframe.resources.providers.ResourceProvider;
import schedframe.resources.units.ResourceUnit;
import schedframe.resources.units.ResourceUnitName;
import schedframe.scheduling.queue.QueueDescription;

public class SchedulerDescription extends ResourceDescription implements ExecutingResourceDescription{

	protected ResourceProvider provider;
	protected List<QueueDescription> accesQueues;
	protected Map<ResourceUnitName, List<ResourceUnit>> resUnits;

	public SchedulerDescription(ResourceProvider provider){
		super(StandardResourceType.ResourceProvider);
		this.id = provider.getProviderId();
		this.provider = provider;
		this.resUnits = new HashMap<ResourceUnitName, List<ResourceUnit>>();
		this.accesQueues = new ArrayList<QueueDescription>(1);
	}
	
	public SchedulerDescription(String id){
		super(StandardResourceType.ResourceProvider);
		this.id = id;
		this.provider = new ResourceProvider(id, null, null, null);
		this.resUnits = new HashMap<ResourceUnitName, List<ResourceUnit>>();
		this.accesQueues = new ArrayList<QueueDescription>(1);
	}
	
	public void addQueueDescription(QueueDescription queue){
		accesQueues.add(queue);
	}
	
	public void addQueuesDescription(List<QueueDescription> queue){
		accesQueues.addAll(queue);
	}
	
	public List<QueueDescription> getAvailableQueues(){
		return this.accesQueues;
	}
	
	public void addResourceUnit(Map<ResourceUnitName, ResourceUnit> allUnits){
		Iterator<ResourceUnit> itr = allUnits.values().iterator();
		while(itr.hasNext()){
			addResourceUnit(itr.next());
		}
	}
	
	public void addResourceUnitList(Map<ResourceUnitName, List<ResourceUnit>> allUnits){
		this.resUnits.putAll(allUnits);
	}

	public ResourceProvider getProvider() {
		return provider;
	}
	
	public void addResourceUnit(ResourceUnit unit) {
		if (this.resUnits == null)
			this.resUnits = new HashMap<ResourceUnitName, List<ResourceUnit>>(1);
		List<ResourceUnit> list = null;
		if (this.resUnits.containsKey(unit.getName())) {
			list = this.resUnits.get(unit.getName());
		} else {
			list = new ArrayList<ResourceUnit>(1);
			this.resUnits.put(unit.getName(), list);
		}
		list.add(unit);
	}
	
	public ResourceUnit getResourceUnit(ResourceUnitName unitName) throws NoSuchFieldException {
		return getResourceUnitList(unitName).get(0);
	}

	public List<ResourceUnit> getResourceUnitList(ResourceUnitName unitName) throws NoSuchFieldException {
		if (resUnits.containsKey(unitName))
			return resUnits.get(unitName);
		else
			throw new NoSuchFieldException("Resource unit " + unitName + " is not available in resource " + this.id);
	}

	public Collection<ResourceUnit> getResourceUnit() {
		if (resUnits == null)
			return null;
		List<ResourceUnit> values = new ArrayList<ResourceUnit>();
		Collection<List<ResourceUnit>> lists = resUnits.values();
		Iterator<List<ResourceUnit>> itr = lists.iterator();

		while (itr.hasNext()) {
			List<ResourceUnit> list = itr.next();
			values.addAll(list);
		}

		return values;
	}

	public Map<ResourceUnitName, List<ResourceUnit>> getResourceUnits() {
		return resUnits;
	}
}
